package Boot.MDELite;

import MDLUtilities.Marquee4Conform;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.Table;
import PrologDB.Tuple;
import java.util.function.Predicate;

public class CategoryConform {
    
    static ErrorReport er;
    
    public static void main(String... args) {
        String expl;
        
        // Step 1: Standard marquee processing
        Marquee4Conform mark = new Marquee4Conform(CategoryConform.class, ".fsm.pl", args);
        String inputFileName = mark.getInputFileName();

        // Step 2: read database and create error handler
        DB db = DB.read(inputFileName);
        Table node = db.getTableEH("node");
        Table edge = db.getTableEH("edge");
        er = new ErrorReport();

        db.StandardConform(er, false);
        // processes these constraints
//        // Step 3: apply constraints
//        // C1: all node ids are unique
//        // C2: all edge ids are unique
//        
//        node.uniqueId(er);
//        edge.uniqueId(er);
//
//        // C3: All cid1 references are legit
//        // C4: All cid2 references are legit
//        edge.isLegit("startid", node, "nid", er);
//        edge.isLegit("endid", node, "nid", er);
        
        // All nodes & edges must have a non blank name
        node.ifThen(g->g.is("text",""), "a blank text field", er);
        edge.ifThen(g->g.is("label",""), "a blank label field", er);


        // node constraints 
        // no two state nodes have the same text
        node.select(r->r.is("ntype","state")).isUnique("text", er);
        // no two edges should have the same non-empty label
        edge.select(r->!r.is("label","")).isUnique("label",er);
        
        // no fsm should have ntype = point
        Predicate<Tuple> ntypeisp = r->r.is("ntype","point");
        
        // only one note that starts with paths, arrows, domains

        Table notes = node.select(n->n.is("ntype","note"));
        eReporter(notes.select(t->t.get("text").startsWith("paths")).count(),"paths");
        eReporter(notes.select(t->t.get("text").startsWith("arrows")).count(),"arrows");
        eReporter(notes.select(t->t.get("text").startsWith("arrows")).count(),"arrows");

        expl = "no points declarations allowed";
        node.ifThen(ntypeisp, expl,er);
        
        // no init, final or point nodes
        for (Tuple t : node.tuples()) {
            String type = t.get("ntype");
            if (type.equals("state") || type.equals("note"))
                continue;
            er.add("node(%s...) %s node not permitted in a category diagram",t.get("nid"),type);
        }

        // End report all errors
        er.printEH(System.out);
    }
    
    static void eReporter(int cnt, String type) {
        switch(cnt) {
            case 0 :  er.add("no note for %s", type); 
            case 1 :  return;
            default : er.add("too many notes for %s only one allowed", type);
        }
    }
}
