package MDLUtilities;

import static MDLUtilities.Error.fileNoExist;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/** standard MDELite Marquee processing */
public class Marquee1In_1DirOut extends MarqueeCommon {

    String outputDirectory;

    /** standard usage:
        Marquee1In_1DirOut mark = new Marquee1In_1DirOut(cls, ".c.d", "JavaDir", args[])
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.outputDirectoryName();
        String appName = mark.getAppName(outputFileName);
     * @param cls -- class object of MDELite application
     * @param inputFilePattern -- one input pattern
     * @param outputDirectory -- one output pattern
     * @param args -- command-line arguments from main(args)
     */
    public Marquee1In_1DirOut(Class cls, String inputFilePattern, String outputDirectory, String args[]) {
        super(cls, inputFilePattern, args);
        this.outputDirectory = outputDirectory;
    }

    /** create a marquee -- I think to passify a whiney compiler */
    public Marquee1In_1DirOut() {
    }

    /**
     * display generic marquee
     * @param msg -- if there is a special error, provide it here; "" otherwise
     */
    public void displayMarquee(String msg) {
        if (msg != null) {
            System.out.format("\n---> %s\n", msg);
        }
        System.out.format("\nUsage: %s X%s [X%s]\n", cls.getName(), inputPattern1, outputDirectory);
        System.out.format("       translate X%s to files in directory X%s\n", inputPattern1, outputDirectory);
        System.out.format("       1st argument must conform to format X%s\n", inputPattern1);
        System.out.format("       2nd argument overrides default output directory (%s)\n", outputDirectory);
        System.exit(0);
    }

    /**
     * @return string of input file name (the first argument)
     */
    @Override
    public String getInputFileName() {
        int len = args.length;
        if (len == 0) {
            displayMarquee(null);
        }
        if (!args[0].contains(".")) //NEW
            return args[0]+inputPattern1;
        if (!args[0].endsWith(inputPattern1)) {
            displayMarquee(args[0] + " submitted as input");
        }
        return args[0];
    }

    /**
     * @return -- output file name (second argument)
     */
    public String outputDirectoryName() {
        String result = "";
        int len = args.length;
        switch (len) {
            case 0 : displayMarquee(null); break;
            case 1 : result = outputDirectory; break;
            default : result = args[1];
        }
        return result;
    }

    /**
     * @param i -- index of command-line argument to return
     * @return argument # i
     */
    public String getArgument(int i) {
        switch (i) {
            case 0:
                return getInputFileName();
            case 1:
                return outputDirectoryName();
            default:
                if (i < 0 || i >= args.length) {
                    displayMarquee("input index " + i + " requested and doesn't exist");
                } else {
                    return args[i];
                }
        }
        return null; // never reaches here; added to pacify whiney compiler
    }

    /**
     * @return PrintStream of outputFile
     */
    public PrintStream getOutputPrintStream() {
        PrintStream out = null;
        String outFile = outputDirectoryName();
        try {
            out = new PrintStream(outFile);
        } catch (FileNotFoundException e) {
            throw Error.toss(fileNoExist, outFile);
        }
        return out;
    }
    
    /** output file names are dir/dir/dir/X.schemaname.pl
     * @return -- name dir/dir/dir/X of output application
     */
    public String getLongAppName() {
        String shorten = outputDirectoryName().replace(outputDirectory, "");
        return shorten;
    }
}
