package MDLUtilities;

/** general Marquee 1 argument programs */
public class Marquee3In_1Out extends MarqueeCommon {
    String[] inputPattern = new String[3];
    String outputPattern;

    /** general Marquee standard usage:
        Marquee3In_1Out mark = new Marquee3In_1Out(THIS.class, ".a.b", ".c.d", ".e.f", "o.p", args);
        String inputFileName1 = mark.getInputFileName(1);
        String inputFileName2 = mark.getInputFileName(2);
        String inputFileName3 = mark.getInputFileName(3);
        String AppName = mark.getAppName(inputFileName2,".c.d"));
     * @param cls -- class object of the Conformance Program
     * @param ipattern1 -- file pattern (".a.b") to match/process
     * @param ipattern2 -- file pattern (".c.d") to match/process
     * @param ipattern3 -- file pattern (".e.f") to match/process
     * @param opattern  -- file pattern ("o.p") to output
     * @param args -- from main(args)
     */
    public Marquee3In_1Out(Class cls, String ipattern1, String ipattern2, String ipattern3, String opattern, String args[]) {
        this.cls = cls;
        this.inputPattern[0] = ipattern1;
        this.inputPattern[1] = ipattern2;
        this.inputPattern[2] = ipattern3;
        this.outputPattern = opattern;
        this.args = args;
        insist(3);
    }
    
    /** to pacify whiney compiler */
    private Marquee3In_1Out() {}
   

    /**
     * display generic marquee for Conformance files
     * @param msg -- specialized error message to print, otherwise ""
     */
    public void displayMarquee(String msg) {
        System.out.format("\nUsage: %s X%s Y%s Z%x (produces) O%s\n", cls.getName(), inputPattern[0], inputPattern[1], inputPattern[2], outputPattern);
        if(msg!=null) {
            System.out.format("\n       %s\n", msg);
        }
        System.exit(0);
    }

    /**
     * @param i - index of parameter, must be greater than 0
     * @return  name of the input file that is to be validated
     */
    public String getInputFileName(int i) {
        if (i==0) {
            displayMarquee("getInputFileName(0) called -- index starts at 1");
        }
        int x = i-1;
        if (!args[x].contains(".")) //NEW
            return args[x]+inputPattern[x];
        if (!args[x].endsWith(inputPattern[x])) {
            displayMarquee(args[x] + " submitted as input expecting '" + inputPattern[x]+"'");
        }
        return args[x];
    }
    
    @Override
    public String getInputFileName() {
        throw new UnsupportedOperationException("Should not be called"); 
    }
    
    @Override
    public String getAppName() {
        throw new UnsupportedOperationException("Should not be called"); 
    }
    
    public String getAppName(String fn,String ipattern) {
        String shorten = fn.replace(ipattern, "");
        String[] arr = Utils.parseFileName(shorten);
        return arr[0];
    }

}
