package Parsing.GeneralPrimitives;

import MDLUtilities.ParsE;
import Parsing.LineParsers.LineToParse;

/** NonEmptyList is a sequence of "node ( comma node )*" */
public class NonEmptyList extends Token { // comma separated tuple values

    // node, comma are objects used as "templates" ala MDELite parsing.
    // skipComma= true means the "comma" token is to be skipped and not saved in 
    // parsing output
    Token node;
    Token comma;
    boolean skipComma;

    /** Standard constructor
     * @param node -- token that appears in a list
     * @param comma -- token separator that is used
     * @param skipComma -- true means comma should be skipped, not saved.
     */
    public NonEmptyList(Token node, Token comma, boolean skipComma) {
        this.node = node;
        this.comma = comma;
        this.skipComma = skipComma;
    }
    
    /** Standard constructor with line to parse
     * @param node -- token that appears in a list
     * @param comma -- token separator that is used
     * @param l -- line to parse
     * @param skipComma -- should comma be skipped or saved?
     */
    public NonEmptyList(Token node, Token comma, LineToParse l, 
            boolean skipComma) {
        super(l);
        this.node = node;
        this.comma = comma;
        this.skipComma = skipComma;
    }

    Token next;
    
    @Override
    public boolean canParse() {
        toBeParsed.AssertNotAtEnd(this);
        next = node.klone(toBeParsed);
        return next.canParse();
    }

    @Override
    public void consumeToken() {
        next.consumeToken();
    }

    @Override
    public void parse() { 
        // remember: canParse was called before parse
        // always parse the first node, from then it can be optional
        // toBeParsed.parseSaveQualified(next);
        next.parse();

        Token c = comma.klone(toBeParsed);
        if (c.canParse()) {
            if (skipComma)
               toBeParsed.parseSkip(c);
            else 
                toBeParsed.parseSave(c);
            next = node.klone(toBeParsed);
            if (next.canParse()) {
                parse();
            } else {
                throw ParsE.toss(ParsE.parseError, toBeParsed.lineno, next.getEmsg() + " expected at >>" + toBeParsed.toParse);
            }
        }
    }

    @Override
    public NonEmptyList klone(LineToParse l) {
        return new NonEmptyList(node, comma, l, skipComma);
    }

    @Override
    public String getEmsg() {
        return node.getEmsg();
    }
}
