package Parsing.GeneralPrimitives.SaveTokens;

import Parsing.GeneralPrimitives.Token;
import Parsing.LineParsers.LineToParse;

/**
 * parse fixed string and saveToken the result
 */
public abstract class FixedStringToken extends Token {

    /**
     * this is the string of a token that is expected
     */
    String TokenString;

    /**
     * this is the string to report in an error
     */
    String emsg;

    boolean saveToken;

    /**
     * Standard constructor
     *
     * @param saveToken - true if token should be output, false otherwise
     */
    public FixedStringToken(boolean saveToken) {
        TokenString = getTokenString();
        emsg = getEmsg();
        this.saveToken = saveToken;
    }

    /**
     * Standard constructor with line to parse
     *
     * @param l -- line to parse
     * @param saveToken - true if token should be output, false otherwise
     */
    public FixedStringToken(LineToParse l, boolean saveToken) {
        super(l);
        TokenString = getTokenString();
        emsg = getEmsg();
        this.saveToken = saveToken;
    }

    /**
     * @return fixed string TokenString to parse
     */
    @Override
    public abstract String getTokenString();

    @Override
    public boolean canParse() {
        toBeParsed.AssertNotAtEnd(this);
        return toBeParsed.startsWith(TokenString);
    }

    @Override
    public void consumeToken() {
        if (saveToken) {
            toBeParsed.returnSave(TokenString.length());
        } else {
            toBeParsed.returnSkip(TokenString.length());
        }
    }

    @Override
    public void parse() {
        if (saveToken) {
            toBeParsed.parseSaveQualified(this);
        } else {
            toBeParsed.parseSkipQualified(this);
        }
    }
}
