package PrologDB;

import java.util.EnumSet;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collector;


/** used as argument to stream collect operation-- to convert streams of tuples
 *  to streams of tables.  Must provide schema of table up front, as it is
  possible that an empty table will be produced.  Calling sequence
  .collect(new TuplesToTable(tableschema))
 */
public class TablesToTable implements Collector<Table,TupleList,Table> {
    static TableSchema empty = new TableSchema("empty","id");
    
    private TableSchema tschema;  // schema of table to produce
    
    /** create table accumulator 
     */
    public TablesToTable() {
        tschema = null;
    }
    
    // See http://blog.indrek.io/articles/creating-a-collector-in-java-8/

    @Override
    public BiConsumer<TupleList, Table> accumulator() {
        return (tl, t) -> tl.addTuples(t.tuples());
    }

    @Override
    public Set<Characteristics> characteristics() {
        return EnumSet.of(Characteristics.UNORDERED);
    }

    @Override
    public BinaryOperator<TupleList> combiner() {
        return TupleList::addTuples;
    }

    @Override
    public Function<TupleList, Table> finisher() {
        return (tl) -> { Tuple t = tl.getFirst();
                         if (t==null) 
                             return new Table(empty,tl);
                         else
                             return new Table(t.getSchema(),tl); };
    }

    @Override
    public Supplier<TupleList> supplier() {
        return TupleList::new;
    }  
}
