package Violett;

import MDLUtilities.Marquee4Conform;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.Table;
import PrologDB.Tuple;
import java.util.function.Predicate;

/** Conformance program for .vpl.pl databases */
public class CategoryConform{
   
    /** Conformance program for .vpl.pl databases
     * @param args X.vpl.pl
     */
    public static void main(String... args) {
        String expl;
        
        // Step 1: Standard marquee processing
        Marquee4Conform mark = new Marquee4Conform(CategoryConform.class, ".fsm.pl", args);
        String inputFileName = mark.getInputFileName();

        // Step 2: read database and create error handler
        DB db = DB.read(inputFileName);
        Table node = db.getTableEH("node");
        Table edge = db.getTableEH("edge");
        ErrorReport er = new ErrorReport();

        db.StandardConform(er, false);
        // handles these constraits
//        // Step 3: apply constraints
//        // C1: all node ids are unique
//        // C2: all edge ids are unique
//        node.isUnique("nid", er);
//        edge.isUnique("eid", er);
//
//        // C3: All cid1 references are legit
//        // C4: All cid2 references are legit
//        edge.isLegit("startid", node, "nid", er);
//        edge.isLegit("endid", node, "nid", er);

        // node constraints 
        // no two state nodes have the same text
        node.select(r->r.is("ntype","state")).isUnique("text", er);
        
        // there are no start and stop nodes
        node.select(r->r.isNot("ntype", "state")).error(er, "node id=%s start and stop nodes are illegal", e->e.getId());
        
        // no two edges should have the same non-empty label
        edge.select(r->!r.is("label","")).isUnique("label",er);
        
        // no fsm should have ntype = point
        Predicate<Tuple> ntypeisp = r->r.is("ntype","point");

        expl = "no points declarations allowed";
        node.ifThen(ntypeisp, expl,er);
        
        // check if multi-input arrows are properly formed
        for (Tuple a : node.select(n->n.is("ntype","state")).tuples()) {
            String nlabel = a.get("text");
            if (nlabel.contains(",")) {
                String[] inputs = nlabel.split(",");
                String nid = a.getId();
                int n = edge.select(e->e.is("endid",nid)).count();
                if (n!=inputs.length) {
                    er.add("node id=%s does not have correct number of arrow inputs",a.getId());
                }
                edge.select(e->e.is("endid",nid) && e.isNot("label",""))
                    .error(er, "node id=%s should have an empty label", e->e.getId());
            }    
        }
//        // every node, except start nodes, have an incoming edge
//        Predicate<Tuple> notPointOrNote = r->!(r.is("ntype","point") || r.is("ntype","note"));
//        Table needsEdge = node.select(notPointOrNote);
//        Table missingInComing = needsEdge.leftAntiSemiJoin("nid", edge, "endid").select(r->!r.is("ntype","init"));
//        missingInComing.stream().forEach(r->er.add("node nid=%s does not have an incoming edge",r.get("nid")));
//        
//        // every node, except final nodes have an outgoing edge
//        Table missingOutGoing = needsEdge.leftAntiSemiJoin("nid",edge,"startid").select(r->!r.is("ntype","final"));
//        missingOutGoing.stream().forEach(r->er.add("node nid=%s does not have an outgoing edge",r.get("nid")));

        // End report all errors
        er.printEH(System.out);
    }
}
