package Yuml;

import MDLUtilities.Marquee4Conform;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.Table;
import PrologDB.Tuple;
import java.util.function.Predicate;

public class ClassConform {
    static Table Box, Assoc;
    
    public static void main(String... args) {
        String expl;
        
        // Step 1: Standard marquee processing
        Marquee4Conform mark = new Marquee4Conform(Yuml.ClassConform.class, ".ypl.pl", args);
        String inputFileName = mark.getInputFileName();

        // Step 2: read database and create error handler
        DB db = DB.read(inputFileName);
        Box = db.getTableEH("yumlBox");
        Assoc = db.getTableEH("yumlAssociation");
        ErrorReport er = new ErrorReport();

        db.StandardConform(er, false);
        // does all this and more
        // Step 3: apply constraints
        // C1: all Box ids are unique
        // C2: all Assoc ids are unique
//        Box.isUnique("id", er);
//        Assoc.isUnique("id", er);
//
//
//        // C3: All box1 references are legit
//        // C4: All box2 references are legit
//        Assoc.isLegit("box1", Box, "id", er);
//        Assoc.isLegit("box2", Box, "id", er);
        
        Box.select(r->!r.is("type","n")).isUnique("name",er);
        // Box constraints 
        // % type = c(class),i(nterface),n(ote).  packages are skipped   
        // table(Box,[id,type,"name","fields","methods",x,y]).
        // C5: if type = i, then fields are empty
        // C6: if type = n, then fields and methods are empty
        // C6': if box is abstract, then type = c
        Predicate<Tuple> typeisi = r->r.is("type","i");
        Predicate<Tuple> typeisn = r->r.is("type","n");
        Predicate<Tuple> typeisc = r->r.is("type","c");
        Predicate<Tuple> BoxIsAbstract = r->r.is("abst","true");
        Predicate<Tuple> fieldsIsEmpty = r->r.is("fields","");
        Predicate<Tuple> methodsIsEmpty = r->r.is("methods","");
        
        expl = "interfaces should have no fields";
        Box.implies(typeisi, fieldsIsEmpty, expl, er);
        
        expl = "notes should have no fields or methods";
        Box.implies(typeisn, fieldsIsEmpty.and(methodsIsEmpty),expl,er);
        
        expl = "has abst='true'; only classes can be abstract";
        Box.implies(BoxIsAbstract, typeisc,expl,er);

        Predicate<Tuple> e1isTriangle = t -> t.is("end1", "^");
        Predicate<Tuple> e2isTriangle = t -> t.is("end2", "^");
        Predicate<Tuple> e1isEmpty = t -> t.is("end1", "");
        Predicate<Tuple> e2isEmpty = t -> t.is("end2", "");
        Predicate<Tuple> r1isEmpty = t -> t.is("role1", "");
        Predicate<Tuple> r2isEmpty = t -> t.is("role2", "");

        expl = "end1=TRIANGLE implies (end2 is empty and role1 is empty and role2 is empty)";
        Assoc.implies(e1isTriangle, e2isEmpty.and(r1isEmpty.and(r2isEmpty)), expl, er);
        Assoc.implies(e2isTriangle, e1isEmpty.and(r1isEmpty.and(r2isEmpty)), expl, er);

        // C11 if end1==TRIANGLE and type1=c and type2=i, then lineStyle== dashed, symmetrically
        Table intf = Box.select(r->r.is("type","i"));
        Table klasses = Box.select(r->r.is("type","c"));
        Table a1 = Assoc.leftSemiJoin("box1", intf, "id");
        Table a2 = a1.leftSemiJoin("box2", klasses, "id");
       
        Predicate<Tuple> isDashed = r -> r.is("lineType", "-.-");
        expl = "linestyle should be dashed";
        a2.implies(e1isTriangle, isDashed, expl, er);
        
        // now symmetrical check
        a1 = Assoc.leftSemiJoin("box1", klasses, "id");
        a2 = a1.leftSemiJoin("box2", intf, "id");
        a2.implies(e2isTriangle, isDashed, expl, er);
        
        // cycle checking: table a2 contains associations only between classes
        // first check for extends cycles

        a1 = Assoc.leftSemiJoin("box1",klasses,"id");
        a2 = a1.leftSemiJoin("box2",klasses,"id");
        
        Table ct = Table.makeCycleTable();
        a2.select(e1isTriangle).forEach(t->addTuple(t,ct,true));
        a2.select(e2isTriangle).forEach(t->addTuple(t,ct,false));
        //ct.sort("par",true).print(System.err);
        ct.cycleCheck(er);
        
        // Finally, see if there are implements cycles 

        a1 = Assoc.leftSemiJoin("box1",intf,"id");
        a2 = a1.leftSemiJoin("box2",intf,"id");
        
        Table ct2 = Table.makeCycleTable();
        a2.select(e1isTriangle).forEach(t->addTuple(t,ct2,true));
        a2.select(e2isTriangle).forEach(t->addTuple(t,ct2,false));
        //ct2.print(System.err);
        ct2.cycleCheck(er);
        
        // End report all errors
        er.printEH(System.out);
    }
                
    private static void addTuple(Tuple t, Table ct, boolean direction) {
        Tuple cycle = new Tuple(ct);
        String box1 = Box.getFirstEH(g->g.is("id",t.get("box1"))).get("name");
        String box2 = Box.getFirstEH(g->g.is("id",t.get("box2"))).get("name");
        if (direction) {
            cycle.setValues(box1, box2);
        } else {
            cycle.setValues(box2, box1);
        }
        ct.add(cycle);
    }
}
