package DML.SchemaTests;

import PrologDB.DBSchema;
import PrologDB.OOSchema;
import RegTest.Utility;
import java.io.File;
import java.io.PrintStream;
import org.junit.Assert;
import org.junit.Test;

public class SchemaCopyTest extends CommonTst {

    static final String testdata = "test/DML/SchemaTests/TestData/";
    static final String correct = "test/DML/SchemaTests/Correct/";
    static final String outfile = "trash/error.txt";
    static final String outfile2 = "trash/error2.txt";

    /* this is composite test -- perhaps developed out of laziness.
       an ooschema is entered as input.  It is flattened and a true
       schema file is produced.  Then the flattened part of the schema is
       copied, and then the schema is produced in whole.
     */
    public SchemaCopyTest() {
    }

    void genericCopySchemaTest(String localFileName) throws Exception {
        // localfilename <dbname>.ooschema.pl
        if (!localFileName.endsWith(".ooschema.pl")) {
            System.err.format("file %s does not end in '.ooschema.pl", localFileName);
            Assert.fail();
            return;
        }
        OOSchema result = null;
        DBSchema copy;
        File schemaFile = new File(testdata + localFileName);
        result = OOSchema.read(schemaFile);

        DBSchema res = result.finishAndPropagateAttributes();
        String flattenedSchema = localFileName.replace(".ooschema.pl", ".schema.pl");
        PrintStream ps = new PrintStream(outfile);  // new PrintStream(testdata + flattenedSchema);   
        res.print(ps);
        ps.println();
        copy = res.copy();
        String newName = copy.getName() + "C";
        copy.rename(newName);
        String fullName = copy.getFullName();
        PrintStream ps2 = new PrintStream(outfile2);
        copy.print(ps2);
        ps2.println();
        ps.close();
        ps2.close();
        Utility.validate(outfile, correct + flattenedSchema, false);
        Utility.validate(outfile2, correct + fullName, false);

    }

    @Test
    public void test1() throws Exception {
        genericCopySchemaTest("violetdb.ooschema.pl");
    }

    @Test
    public void test2() throws Exception {
        genericCopySchemaTest("inh.ooschema.pl");
    }

    @Test
    public void test3() throws Exception {
        genericCopySchemaTest("complex.ooschema.pl");
    }

    @Test
    public void test4() throws Exception {
        genericCopySchemaTest("hierarchy.ooschema.pl");
    }
}
